//=============================================================================
//  MuseScore
//  Music Composition & Notation
//
//  DAd Seagull Merlin Strum Stick - Check For Available Notes
//  Copyright (C) 2016 Jeffrey Edwards
//
//  created from / based on:
//  Note Names Plugin
//  Copyright (C) 2012 Werner Schweer
//  Copyright (C) 2013, 2014 Joachim Schmitz
//  Copyright (C) 2014 Jörn Eichler
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2
//  as published by the Free Software Foundation and appearing in
//  the file LICENCE.GPL
//
//  Thanks to the makers of the Note Names Plugin
//  for making this possible
//
//  Interpretation:
//  This plugin is a quick way to check whether the notes in the score
//  are available on the Seagull Merlin.
//  This plugin will add a stack of three numbers below each note.
//  The top number is the fingering position on the high D (D4) string.
//  The middle number is the fingering position on the middle (A3) string.
//  The bottom number is the fingering position on the low D (D3) string.
//  An "X" indicates that the note is not available on its respective string.
//
//  Horizontal position of gracenotes must be adjusted manually.
//
// Revisions for Version 2:
// 1. changed order of strings so that high "d" is at top of list (reverse of dulcimer tab)
//=============================================================================

import QtQuick 2.0
import MuseScore 1.0

MuseScore {
   version: "2.0"
   description: "This plugin checks whether a note is playable by the DAd Merlin"
   menuPath: "Plugins.Seagull Merlin DAd - Check Notes"

//=============================================================================
   function getD3string (note) {
      //notes on D3(#3) string
      if(note.pitch == 50){return("0");} //D
      if(note.pitch == 51){return("X");} //D# not available
      if(note.pitch == 52){return("1");} //E
      if(note.pitch == 53){return("X");} //F  not available
      if(note.pitch == 54){return("2");} //F#
      if(note.pitch == 55){return("3");} //G
      if(note.pitch == 56){return("X");} //G# not available
      if(note.pitch == 57){return("4");} //A
      if(note.pitch == 58){return("X");} //A# not available
      if(note.pitch == 59){return("5");} //B
      if(note.pitch == 60){return("X");} //C  not available
      if(note.pitch == 61){return("6");} //C#
      if(note.pitch == 62){return("7");} //D
      if((note.pitch <  50) || (note.pitch > 62)) {return("X");} //if note is outside playable range for this string
   }//end function getD3string 
//=============================================================================
   function getA3string (note) {
      //notes on A3(#2) string
      if(note.pitch == 57){return("0");} //A
      if(note.pitch == 58){return("X");} //A# not available
      if(note.pitch == 59){return("1");} //B
      if(note.pitch == 60){return("X");} //C  not available
      if(note.pitch == 61){return("2");} //C#
      if(note.pitch == 62){return("3");} //D
      if(note.pitch == 63){return("X");} //D# not available
      if(note.pitch == 64){return("4");} //E
      if(note.pitch == 65){return("X");} //F  not available
      if(note.pitch == 66){return("5");} //F#
      if(note.pitch == 67){return("X");} //G  not available
      if(note.pitch == 68){return("6");} //G#
      if(note.pitch == 69){return("7");} //A
      if((note.pitch <  57) || (note.pitch > 69)) {return("X");} //if note is outside playable range for this string
   }//end function getA3string 
//=============================================================================
   function getD4string (note) {
      //notes on D4(#1) string
      if(note.pitch == 62){return("0");} //D
      if(note.pitch == 63){return("X");} //D# not available
      if(note.pitch == 64){return("1");} //E
      if(note.pitch == 65){return("X");} //F  not available
      if(note.pitch == 66){return("2");} //F#
      if(note.pitch == 67){return("3");} //G
      if(note.pitch == 68){return("X");} //G# not available
      if(note.pitch == 69){return("4");} //A
      if(note.pitch == 70){return("X");} //A# not available
      if(note.pitch == 71){return("5");} //B
      if(note.pitch == 72){return("X");} //C  not available
      if(note.pitch == 73){return("6");} //C#
      if(note.pitch == 74){return("7");} //D
      if((note.pitch <  62) || (note.pitch > 74)) {return("X");} //if note is outside playable range for this string
   }//end function getD4string 

//=============================================================================

   onRun: {
      if (typeof curScore === 'undefined')
         Qt.quit();
      var cursor = curScore.newCursor();
      var startStaff;
      var endStaff;
      var endTick;
      var fullScore = false;
      cursor.rewind(1);
      if (!cursor.segment) { // no selection
         fullScore = true;
         startStaff = 0; // start with 1st staff
         endStaff  = curScore.nstaves - 1; // and end with last
      } else {
         startStaff = cursor.staffIdx;
         cursor.rewind(2);
         if (cursor.tick == 0) {
            // this happens when the selection includes
            // the last measure of the score.
            // rewind(2) goes behind the last segment (where
            // there's none) and sets tick=0
            endTick = curScore.lastSegment.tick + 1;
         } else {
            endTick = cursor.tick;
         }
         endStaff   = cursor.staffIdx;
      }

      for (var staff = startStaff; staff <= endStaff; staff++) {
         for (var voice = 0; voice < 4; voice++) {
            cursor.rewind(1); // beginning of selection
            cursor.voice    = voice;
            cursor.staffIdx = staff;

            if (fullScore)  // no selection
               cursor.rewind(0); // beginning of score

            while (cursor.segment && (fullScore || cursor.tick < endTick)) {
               if (cursor.element && cursor.element.type == Element.CHORD) {
              
                  var notes = cursor.element.notes;
                  var text = newElement(Element.STAFF_TEXT);
                  var text_D3 = getD3string(notes[0]); //check low D string
                  var text_A3 = getA3string(notes[0]); //check A string
                  var text_D4 = getD4string(notes[0]); //check high D string
                  text.text = "<font size=\"12\"/>";              
                  text.text = text.text + text_D4 + "\n" + text_A3 + "\n" + text_D3;
                  if((text_D3 == "X")&&(text_A3 == "X")&&(text_D4 == "X")){text.color = "red";}
                  text.pos.y = 9;
                  cursor.add(text);

                  var graceChords = cursor.element.graceNotes;
                  // iterate through all grace chords
                  for (var i = 0; i < graceChords.length; i++) {
                     var notes = graceChords[i].notes;
                     // there seems to be no way of knowing the exact horizontal pos.
                     // of a grace note, so we have to guess:
                     // we need a better way of determining the Xposition of the gracenote
                     // until then adjust position manually
                     var posX = 2.5 * (graceChords.length - i);
                     var text = newElement(Element.STAFF_TEXT);
                     var text_D3 = getD3string(notes[0]); //check low D string
                     var text_A3 = getA3string(notes[0]); //check A string
                     var text_D4 = getD4string(notes[0]); //check high D string
                     text.text = text_D3 + "\n" + text_A3 + "\n" + text_D4;
                     text.pos.x = posX;
                     text.text = "<font size=\"9\"/>";              
                     text.text = text.text + text_D4 + "\n" + text_A3 + "\n" + text_D3;
                     text.pos.y = 9;
                     cursor.add(text);
                  }//end for

               } // end if CHORD

                  cursor.next();
            } // end while segment
			
         } // end for voice
      } // end for staff
      Qt.quit();
   } // end onRun
} //end MuseScore
